# ==================================================================
# Plugins::Thumb_Images - Auto Generated Program Module
#
#   Plugins::Thumb_Images
#   Author  : Andy Newby
#   Version : 7.0.2
#   Updated : 8th Jan 2013
#
# ==================================================================
#

package Plugins::Thumb_Images;
# ==================================================================

use strict;
use GT::Base;
use GT::Plugins qw/STOP CONTINUE/;
use Links qw/:objects/;
use Links::Plugins;
use GT::SQL::File;
use File::Find;
use Image::Magick;
use Image::Size;
use File::Copy;

# Inherit from base class for debug and error methods
@Plugins::Thumb_Images::ISA = qw(GT::Base);

 
my $debug = 0;

sub _load_config {
#######################################################
# Call _load_config(), as load_config() seems to be taken by another module...

	# Get all the other different sizes, for the other types...
	my $PCFG = Links::Plugins::get_plugin_user_cfg ('Thumb_Images');

		# WayToConvert => command | module
		# links_fields => fields that need doing

	$Thumb_Images::CFG    			= $PCFG;		
		
	my $other_sizes;
	   $other_sizes->{'Link'}    	= {};
	   $other_sizes->{'Category'} 	= {}; 
	   $other_sizes->{'Review'}    	= {};
		
	my @links_fields = qw//;
	my %links_fields_seen;
	if ($PCFG->{links_fields}) {
		my @split = split /\n/, $PCFG->{links_fields};
		foreach (@split) {
			my ($field,$size,$type) = split /\:\:/, $_;
			$other_sizes->{'Link'}->{$field}->{$type} = $size;
			if (!$links_fields_seen{$field}) {
				$links_fields_seen{$field} = 1;
				push @links_fields, $field;
			}
		}
	}
	
	$Thumb_Images::LINKS::FIELDS 	= \@links_fields;
	$Thumb_Images::LINKS::SIZES  	= $other_sizes->{'Link'};
	
	
	my @cat_fields = qw//;
	my %cat_fields_seen;
	if ($PCFG->{category_fields}) {
		my @split = split /\n/, $PCFG->{category_fields};
		foreach (@split) {
			my ($field,$size,$type) = split /\:\:/, $_;
			$other_sizes->{'Category'}->{$field}->{$type} = $size;
			if (!$cat_fields_seen{$field}) {
				$cat_fields_seen{$field} = 1;
				push @cat_fields, $field;
			} 
		}
	}

	$Thumb_Images::CATS::FIELDS 	= \@cat_fields;
	$Thumb_Images::CATS::SIZES  	= $other_sizes->{'Category'};	
	
	my @review_fields = qw//;
	my %review_fields_seen;
	if ($PCFG->{review_fields}) {
		my @split = split /\n/, $PCFG->{review_fields};
		foreach (@split) {
			my ($field,$size,$type) = split /\:\:/, $_;
			$other_sizes->{'Review'}->{$field}->{$type} = $size;
			if (!$review_fields_seen{$field}) {
				$review_fields_seen{$field} = 1;
				push @review_fields, $field;
			}
		}
	}

	$Thumb_Images::REVIEW::FIELDS 	= \@review_fields;
	$Thumb_Images::REVIEW::SIZES  	= $other_sizes->{'Review'};		
	
}

sub check_if_should_run {

	my @args = @_;
	
	#print $IN->header;
	#use Data::Dumper;
	#print Dumper(@args);
	
	if (!$args[0]->{error} && $args[0]->{ID}) {
		$CUSTOM::LID = $args[0]->{ID}; # set this so we know which one to run!
	} else {
		# dont run, as it looks like there was an error!
	}
	return @args;
}


sub Cleanup {

	print $IN->header;

	print q|
		<h2>Purge and Resize Images</h2>
		
		<p>The following feature is only recommended for ADVANCED USERS! If you are not confident on how to back up your images, please email andy@ultranerds.co.uk before doing this - I would rather give you advice on how to properly backup, just in case you need to restore!<p>

		<p>This will remove the thumbnailed images for each of the links/reviews/categories. They will then be re-created. This is so you can "resize" them if needed (i.e you have changed the size of your thumbnails, or just made additional sizes)</p>

		<p>If you are happy to run this, <a href="./admin.cgi?do=plugin&plugin=Thumb_Images&func=Cleanup_Do" onclick="return confirm('Are you sure you have made a backup of your Files and Database? Just incase you need to restore it!')">then please click this link</a></p>
		
	|;
	
}

sub Cleanup_Do {

	_load_config(); # make sure we do this so we get all our options and cfg!

	print $IN->header;
	
	$Plugins::Thumb_Images::show = 1;
	
	my $delim;
	if ($IN->param('shell')) {
		$delim = "\n";
	} else {
		$delim = "<br />";
	}
	
	#########################
	# LINKS 
	#########################
	
	print qq|<h2>Processing Links...</h2>|;
	
	# Lets get the paths... in case they use the same paths, we only wanna run it ONE TIME per path!
	my $paths = {};
	foreach (@{$Thumb_Images::LINKS::FIELDS}) {
		print "Working on: $_ $delim ";
		my $path = $DB->table('Links')->cols->{$_}->{'file_save_in'};
		$paths->{$path} = 1;	
	}
	
	map { 
		print "Cleaning Link Folder, $_ $delim <br />";
		foreach my $fid (0 .. 9) {
			print "Sub-Folder: $fid <Br />";
			find(\&clean_old_files, "$_/$fid");
		}
	} keys %$paths;

	# Now we've removed all the "old" smaller sized images, lets remake just the ones we want!

	my $tbl = $DB->table('Links');
	   $tbl->select_options("ORDER BY ID");
	my $sth = $tbl->select( ['ID'], { isValidated => "Yes" } );
	
	my $total = $sth->rows;	
	
	my $i = 0;
	while (my $id = $sth->fetchrow) {
		$i++;
		print "Doing $id ($i / $total) $delim";
		do_shrink_link($id);
	}

	
	#########################
	# CATEGORIES 
	#########################
	
	print qq|<h2>Processing Categories...</h2>|;
	
	# Lets get the paths... in case they use the same paths, we only wanna run it ONE TIME per path!
	my $paths = {};
	foreach (@{$Thumb_Images::CATS::FIELDS}) {
		print "Working on: $_ $delim ";
		my $path = $DB->table('Category')->cols->{$_}->{'file_save_in'};
		$paths->{$path} = 1;	
	}
	
	map { 
		print "Cleaning Category Folder, $_ $delim <br />";
		foreach my $fid (0 .. 9) {
			print "Sub-Folder: $fid <Br />";
			find(\&clean_old_files, "$_/$fid");
		}
	} keys %$paths;
	
	# Now we've removed all the "old" smaller sized images, lets remake just the ones we want!
	my $tbl = $DB->table('Category');
	   $tbl->select_options("ORDER BY ID");
	my $sth = $tbl->select( ['ID'] );
	
	my $total = $sth->rows;	
	
	my $i = 0;
	while (my $id = $sth->fetchrow) {
		$i++;
		print "Doing $id ($i / $total) $delim";
		do_shrink_cat($id);
	}
	
	
	#########################
	# CATEGORIES 
	#########################
	
	print qq|<h2>Processing Reviews...</h2>|;
	
	# Lets get the paths... in case they use the same paths, we only wanna run it ONE TIME per path!
	my $paths = {};
	foreach (@{$Thumb_Images::REVIEW::FIELDS}) {
		print "Working on: $_ $delim ";
		my $path = $DB->table('Category')->cols->{$_}->{'file_save_in'};
		$paths->{$path} = 1;	
	}
	
	map { 
		print "Cleaning Review Folder, $_ $delim <br />";
		foreach my $fid (0 .. 9) {
			print "Sub-Folder: $fid <Br />";
			find(\&clean_old_files, "$_/$fid");
		}
	} keys %$paths;
	
	# Now we've removed all the "old" smaller sized images, lets remake just the ones we want!
	my $tbl = $DB->table('Reviews');
	   $tbl->select_options("ORDER BY ReviewID");
	my $sth = $tbl->select( ['ReviewID'] );
	
	my $total = $sth->rows;	
	
	my $i = 0;
	while (my $id = $sth->fetchrow) {
		$i++;
		print "Doing $id ($i / $total) $delim";
		do_shrink_review($id);
	}
		
	
	
	
}


sub do_shrink_link {

	print $IN->header;

	my $LinkID = $_[0];
	my $link = $DB->table("Links")->get( { ID => $LinkID } ) || {};
	
	#return; 
	
	if ($link->{ID}) {
	
		foreach my $field (@{$Thumb_Images::LINKS::FIELDS}) {	
			
			if ($link->{$field}) {
			
				#print qq|FIELD: $field \n|;
			
				my $read_path = get_path($link->{ID},$field,undef,"Links","path");
			
				if ($read_path !~ /jpg|gif|png$/i) {
					next; 
				}
			
				if (-e "$read_path") {

					# Be sure to rename them if they dont match criteria!
					#print "Checking this is ok: '$link->{$field}' <br />";
					if ($link->{$field} !~ /^[a-z0-9_\-\.]+$/i || $read_path =~ /\%2(7|8|9)/ ) {
						($link->{$field},$read_path) = rename_file($link->{ID},$link->{$field},$read_path,$field,'Links')
					}
		
					# Create alternative sizes, if needed
					my $alternative = $Thumb_Images::LINKS::SIZES->{$field};
					map {
						my $write_path = get_path($link->{ID},$field,$_,"Links","path");					
						create_thumb($read_path,$write_path,$alternative->{$_});
					} keys %$alternative;
					
				} else {
					# We couldn't find it for some reason - weird!
					print qq|Unknown error! ($read_path doesnt exist)\n|;
				}
				
			}
		}
	
	} else {
		
		print "Ack - not a valid link! \n" if $Plugins::Thumb_Images::show;
		return;
	
	}
	
	

}


sub do_shrink_cat {

	print $IN->header;

	my $CatID = $_[0];
	my $cat = $DB->table("Category")->get( { ID => $CatID } ) || {};

	if ($cat->{ID}) {
	
		foreach my $field (@{$Thumb_Images::CATS::FIELDS}) {	
			
			if ($cat->{$field}) {
			
				#print qq|FIELD: $field \n<br />|;
			
				my $read_path = get_path($cat->{ID},$field,undef,"Category","path");
			
				if ($read_path !~ /jpg|gif|png$/i) {
					next; 
				}
			
				if (-e "$read_path") {

					# Be sure to rename them if they dont match criteria!
					#print "Checking this is ok: '$cat->{$field}' <br />";
					if ($cat->{$field} !~ /^[a-z0-9_\-\.]+$/i || $read_path =~ /\%2(7|8|9)/ ) {
						($cat->{$field},$read_path) = rename_file($cat->{ID},$cat->{$field},$read_path,$field,'Category')
					}

					# Create alternative sizes, if needed
					my $alternative = $Thumb_Images::CATS::SIZES->{$field};
					map {
						my $write_path = get_path($cat->{ID},$field,$_,"Category","path");					
						create_thumb($read_path,$write_path,$alternative->{$_});
					} keys %$alternative;
					
				} else {
					# We couldn't find it for some reason - weird!
					print qq|Unknown error! ($read_path doesnt exist)\n|;
				}
			
			}
			
		}
	
	} else {
		
		print "Ack - not a valid cat! \n" if $Plugins::Thumb_Images::show;
		return;
	
	}

}

sub do_shrink_review {

	print $IN->header;

	my $ReviewID = $_[0];
	my $review = $DB->table("Reviews")->get( { ReviewID => $ReviewID } ) || {};

	#print "GOT: $ReviewID ";	
	#use Data::Dumper;
	#print Dumper($Thumb_Images::REVIEW::FIELDS,$review);
	
	if ($review->{ReviewID}) {
	
		foreach my $field (@{$Thumb_Images::REVIEW::FIELDS}) {	

			if ($review->{$field}) {

				my $read_path = get_path($review->{ReviewID},$field,undef,"Reviews","path");
			
				if ($read_path !~ /jpg|gif|png$/i) {
					next; 
				} 
			
				if (-e "$read_path") {

					# Be sure to rename them if they dont match criteria!
					#print "Checking this is ok: '$review->{$field}' <br />";
					if ($review->{$field} !~ /^[a-z0-9_\-\.]+$/i || $read_path =~ /\%2(7|8|9)/ ) {
						($review->{$field},$read_path) = rename_file($review->{ReviewID},$review->{$field},$read_path,$field,'Reviews')
					}

					# Create alternative sizes, if needed
					my $alternative = $Thumb_Images::REVIEW::SIZES->{$field};
					map {
						my $write_path = get_path($review->{ReviewID},$field,$_,"Reviews","path");					
						create_thumb($read_path,$write_path,$alternative->{$_});
					} keys %$alternative;
					
				} else {
					# We couldn't find it for some reason - weird!
					print qq|Unknown error! ($read_path doesnt exist)\n|;
				}
				
			}
			
		}
	
	} else {
		
		print "Ack - not a valid review! \n" if $Plugins::Thumb_Images::show;
		return;
	
	}

}


sub rename_file {
# If the image doesn't match up to what we expect (a-z0-9
	
	my ($ID,$filename,$read_path,$field,$table) = @_;
	
	my @tmp = split /\//, $read_path;
	pop @tmp;
	my $path = join("/", @tmp);
	
	my $new_filename = CORE::time() . "-$ID." . lc((reverse split /\./, $read_path)[0]);
	my $new_path 	  = "$path/$new_filename";
	
	$read_path =~ s/'/\\\'/g; # fix if we have ' in the name!!!
	
	#print "GOT NEW FILE: $new_filename at $new_path for $field \n";
	
	copy($read_path,$new_path);
	my $f = GT::SQL::File->open($new_path);
	
	if ($table eq "Reviews") {
		$DB->table($table)->update( { $field => $f }, { ReviewID => $ID }  ) || die $GT::SQL::error;		
	} else {
		$DB->table($table)->update( { $field => $f }, { ID => $ID }  ) || die $GT::SQL::error;		
	}
	
	return ($new_filename,$new_path);
}	
	
sub create_thumb {

	my ($read_path,$write_path,$dimensions) = @_;
	
	my $fsize = -s $read_path;
	if ($fsize > 0) { 
	
		if ($Thumb_Images::CFG->{WayToConvert} eq "command") { 
	
			my ($x,$y) = imgsize($read_path);
			my ($x_resize,$y_resize) = split /x/, $dimensions;
	
			my ($x,$y) = imgsize($read_path);
			my ($x_resize,$y_resize) = split /x/, $dimensions;
			
			# Resize if over 1000x1000 px - as thats too large to need and just wastes space!
			if ($x > 1000 || $y > 1000) {
				my $command = qq|convert '$read_path' -resize 1000x1000\\\> '$read_path'|;
				`$command`;
			}	
	
			my $command = qq|convert '$read_path' -resize $dimensions\\\> '$write_path'|;
			`$command`;
			
		
		} else {
		
			my ($x,$y) = imgsize($read_path);
			my ($x_resize,$y_resize) = split /x/, $dimensions;
			
			# Resize if over 1000x1000 px - as thats too large to need and just wastes space!
			if ($x > 1000 || $y > 1000) {
				my $image = Image::Magick->new;
				my $x = $image->Read($read_path);
				if ($x) { print "Error: $x"; exit; }

				$x = $image->Scale(geometry=>"1000x1000");
				if ($x) { print "Error: $x"; exit; }

				$x = $image->Write(filename=>$read_path);
				if ($x) { print "Error: $x"; exit; }							
			}
			
			#print qq|Reading: $read_path , and original: $read_path , gonna resize to $dimensions <br /> \n|;
			
			 my $image = Image::Magick->new;
			 my $x = $image->Read($read_path);
			 if ($x) { print "Error: $x"; exit; }

			 if ($x > $x_resize || $y > $y_resize) {
			   $x = $image->Scale(geometry=>$dimensions);
			   if ($x) { print "Error: $x"; exit; }
			 }
	 
			 $x = $image->Write(filename=>$write_path);
			 if ($x) { print "Error: $x"; exit; }				
			
			#print STDERR "RUNNING: $command \n";
		}
		
	} else {
	
		print qq|>>> ERROR: Invalid size for $read_path ! ($fsize) <<< \n|;
	
	}

}

sub add_link {

	my ($p) = @_;

	_load_config(); # make sure we do this so we get all our options and cfg!
	
	if ($ENV{HTTP_REFERER} =~ /add\.cgi/) {
		if ($CFG->{build_auto_validate} == 1 || $CFG->{build_auto_validate} == 2) {
			do_shrink_link($p);	
		} else {
			# Dont do anything.. as we will do it in validate!
		}
	} else {
		do_shrink_link($p);
	}
	
	return @_;

}



sub admin_modify_link {

	#print $IN->header;
	#print "In admin_modify_link <br />";

	_load_config(); # make sure we do this so we get all our options and cfg!	
	do_shrink_link($CUSTOM::LID);
	
	return @_;

}


sub user_modify_link {

	#print $IN->header;
	#print "In user_modify_link <br />";
	
	if ($_[0]->{error}) { return @_; } # catch any errors
	
	_load_config(); # make sure we do this so we get all our options and cfg!
	do_shrink_link($_[0]->{LinkID});
	
	return @_;

}


sub validate_link {

	#print $IN->header;
	#print "GOT: $CUSTOM::LID ";
	#print "vals passed: ";
	#use Data::Dumper;
	#print Dumper(@_);
	
	if ($CUSTOM::LID) {
		_load_config(); # make sure we do this so we get all our options and cfg!
		do_shrink_link($CUSTOM::LID);
	}
	
	return @_;

}


sub add_category {
	
	#print $IN->header;
	#use Data::Dumper;
	#print Dumper(@_);

	my @args = @_;
	
	if (!$args[0]->{error} && $args[0]->{ID}) {
		$CUSTOM::LID = $args[0]->{ID}; # set this so we know which one to run!
		do_shrink_cat($CUSTOM::LID);
	}
	
	return @_;	
	
}


sub modify_category {

	if ($CUSTOM::LID) {
		_load_config(); # make sure we do this so we get all our options and cfg!
		do_shrink_cat($CUSTOM::LID);
	}
	
	return @_;

}

sub review_add {
# =================================

	my @args = @_;

	# If we are set as "All reviews require admin/editor approval (recommended).", we wanna return - cos there is nothing to do here
	if ($CFG->{review_auto_validate} == 0) { return @_ }
	
	if ($args[0]->{ReviewID}) {
		_load_config(); # make sure we do this so we get all our options and cfg!
		$CUSTOM::LID = $args[0]->{ReviewID}; # set this so we know which one to run!
		do_shrink_review($CUSTOM::LID);
	}
	
	return @_;	

}

sub validate_review {
	
	my @args = @_;
	
	if ($args[0]->{ReviewID}) {
		_load_config(); # make sure we do this so we get all our options and cfg!
		$CUSTOM::LID = $args[0]->{ReviewID}; # set this so we know which one to run!
		do_shrink_review($CUSTOM::LID);
	}
	
	return @_;	
	
}

sub update_this_review {
# This only gets run if we have auto-validate enabled (otherwise, they just get validated in validate_review() above)
	
	my @args = @_;

	# ONLY run this when we have "auto validate" set... otherwise it will thumb even before it gets validated
	if ($CFG->{review_auto_validate} != 1) { return @args; }
	
	if ($args[0]->{ReviewID}) {
		_load_config(); # make sure we do this so we get all our options and cfg!
		$CUSTOM::LID = $args[0]->{ReviewID}; # set this so we know which one to run!
		do_shrink_review($CUSTOM::LID);
	}
	return @args;
	
}

sub get_path {
# Called to grab either a URL, or path. Can also pass in the size paramater...
# get_path($ID,Field,size,table,url)

	my ($ID,$field,$size,$table,$url_or_path) = @_;

    $table 	||= 'Links';
	$url_or_path ||= "url";
	
	# make sure a fieldname and ID are provided...
	if (!$ID || !$field) { return; }

	# get the actual path to where the file is/will be saved...

    my $path = $DB->table($table)->cols->{$field}->{$url_or_path eq "url" ? 'file_save_url' : 'file_save_in'};

	$path =~ s,/$,,; # get rid of trailing / at end of $path

	
   # now we have the highest ID, lets get the image stuff for this link.
        
	my $rel_path;
	my $image_details = $DB->table($table."_Files")->select( { ForeignColName => $field, ForeignColKey => $ID } )->fetchrow_hashref;
	
	my $folderid = (reverse split //, $image_details->{ID})[0];
	
	if (!$image_details->{ID}) { 
		return; # doesn't seem to exist for some reason - so return!
	}
	
	if ($url_or_path eq "url") {
		if (!$size) {
			$rel_path = qq|$image_details->{File_URL}/$folderid/$image_details->{ID}-$image_details->{File_Name}|; 
		} else {
			$rel_path = qq|$image_details->{File_URL}/$folderid/$size-$image_details->{ID}-$image_details->{File_Name}|; 
		}

	} else {
		if (!$size) {
			$rel_path = qq|$image_details->{File_Directory}/$folderid/$image_details->{ID}-$image_details->{File_Name}|; 
		} else {
			$rel_path = qq|$image_details->{File_Directory}/$folderid/$size-$image_details->{ID}-$image_details->{File_Name}|; 
		}
	}
	
 
	$rel_path =~ s|([^\/]+)$|GT::CGI->escape( $1 )|e; # we shouldn't need to do this - but lets anyway!

	$rel_path =~ s/'/%27/g; 
	$rel_path =~ s/\(/%28/g; 
	$rel_path =~ s/\)/%29/g; 

	return $rel_path;

}



sub clean_old_files {
		
#	if (-d $File::Find::name ) { next; } 		

	if ($_ !~ /^\d+\-/) {
		if ($_ =~ /.(png|jpe?g|gif)$/i) {
			print "Removing: $_ <br />";	
			unlink($File::Find::name);
		}
	} 

} 



sub Test_Config {

	_load_config(); # make sure we do this so we get all our options and cfg!

	print $IN->header;

	use Data::Dumper;
	
	print "LINK Fields: <pre>" . Dumper($Thumb_Images::LINKS::FIELDS) . "</pre><br />";
	print "LINK Sizes:: <pre>" . Dumper($Thumb_Images::LINKS::SIZES) . "</pre><br />";
	
	print "<hr>";
	
	print "CATEGORY Fields: <pre>" . Dumper($Thumb_Images::CATS::FIELDS) . "</pre><br />";
	print "CATEGORY Sizes:: <pre>" . Dumper($Thumb_Images::CATS::SIZES) . "</pre><br />";

	print "<hr>";
	
	print "REVIEW Fields: <pre>" . Dumper($Thumb_Images::REVIEW::FIELDS) . "</pre><br />";
	print "REVIEW Sizes:: <pre>" . Dumper($Thumb_Images::REVIEW::SIZES) . "</pre><br />";
	
	print "<hr>";
	
	print "PLUGIN Config: <pre>" . Dumper($Thumb_Images::CFG) . "</pre><br />";
	
}

sub test {
	#do_shrink_link(9102);
}

sub Readme {

	print $IN->header;
	

	print q|
<!DOCTYPE html>
	<html lang="en">
		<head>
			<title>Thumb Images Readme</title>
			<link href="//netdna.bootstrapcdn.com/twitter-bootstrap/2.2.2/css/bootstrap-combined.min.css" rel="stylesheet">
		
		</head>
		<body>
		
			<div class="container-fluid">
				<h2>Thumb_Images Readme</h2>
				<div class="row-fluid">
					<div class="span3">
						<ul class="nav nav-tabs nav-stacked">
							<li><a href="#installation">Installation</a></li>
							<li><a href="#addfields">Adding image fields to your system</a></li>
							<li><a href="#fieldsetup">Setting up fields</a></li>
							<li><a href="#templates">Templates</a></li>
							<li><a href="#support">Support</a></li>
						</ul>
					</div>
					<div class="span9">
						
						<dl class="dl-horizontal">
							<dt>Version:</dt>
							<dd>7.0.2</dd>
							
							<dt>Last Updated:</dt>
							<dd>8th Jan 2013</dd>
							
							<dt>Author:</dt>
							<dd>Andy Newby</dd>
														
						</dl>
						
						<h3>Installation</h3>
						<a name="installation"> </a>
						
						<p>You have actually already installed this plugin :) Now you just need to setup your image fields (adding them to your GLinks installation if required, <a href="#addfields">info here</a>)</p>
						
						<h3>Adding image fields</h3>
						<a name="addfields"> </a>
						
						<p>Adding new fields is MUCH simpler in this new version! First goto <b>Plugins > Thumb_Images > Setup</b></p>
						
						<p>You now should see 3 TEXTAREA boxes. These are where your field settings go. <b>IMPORTANT:</b> You don't have to enter values for all of them... only the bits you are gonna use!</p>
						
						<p>Lets take an example image field in the "Links" table called "SiteLogo". We want to make 3 versions - small, med and large... Here is an example of what you could use:</p>
			
						<pre>SiteLogo::200x200::small
SiteLogo::300x300::med
SiteLogo::400x400::large</pre>
						
						<p>The above would create 3 versions of the image (the original is left in its original size, in case you need to re-size them)</p>
						
						<p>IMPORTANT: If you ever change the sizes in here, or just want to add a new image size - just run the <b>Plugins > Thumb_Images > Cleanup</b> option. This will then ask you to confirm running it. Ideally you want to back up the site and your mySQL tables *just in case* before doing this. Nothing bad should happen, but it will wipe out the small-xx med-xx images before re-creating them (this just removes old images and starts fresh for the rebuild)</p>
						
						<h3>Setting up the fields</h3>
						<a name="fieldsetup"> </a>
						
						<p>Adding a new field is actually pretty simple. Just follow these steps:</p>
						
						<ol>
							<li>If you were to add a new image for the "Links" table, got <b>Database > Links > Properties > Add Field</b></li>
							<li>For the values use stuff like below:
								<ul>
									<li><b>Column Name:</b> Your_Field_Name</li>
									<li><b>Column Type:</b> CHAR</li>
									<li><b>Column Size:</b> 255</li>
									<li><b>Form Display:</b> Value to show</li>
									<li><b>Form Type:</b> FILE</li>
									<li><b>File Save Location:</b> /home/user/domain.com/public_html/static/images</li>
									<li><b>File Save URL:</b> http://www.yoursite.com/static/images</li>
									<li><b>File Maximum Size:</b> 30000000</li>
									<li><b>Form Regex:</b> ^(?:\|.*\.(?i:jpg\|jpeg\|jpe\|gif))$</li>
								</ul>
							</li>
							<li>Click the "<b>Add Fields to Links</b>" button</li>
							<li>You now need to add the HTML into include_form.html (also be sure to set the <b>enctype="multipart/form-data"</b> bit in your add.html and modify.html <form> bits!)
							To create valid HTML for uploading files you can make use of one of my free plugins: <a href="http://www.ultranerds.co.uk/Products/Plugins/Gossamer_Links_Plugins/Free_Plugins/GLinks_3_0_x/MakeFormHTML_L273/" target="_blank">MakeFormHTML</a>
							</li>
							<li>Test the file upload system BEFORE setting the new field up in the plugin (this just helps proove your done it right, before telling me the plugin doesn't work <G>)</li>
							<li>If you're happy that it works, you need to add the new field into the Thumb_Images plugin settings (<a href="#addfields">see here</a>)</li>
						</ol>
						
						<h3>Templates</h3>
						<a name="templates"> </a>
						
						<p>Using this new version is far simpler than previous versions :) You simply call it using:</p>
						
						<pre><%Plugins::Thumb_Images::get_path($ID,Field,size,table,url)%></pre>
						
						<p>The "ID" is different depending on which table you are working with. Below is the key:</p>
						
						<table class="table table-striped">
							<thead>
								<tr>
									<th>Table</th>
									<th>Value to use</th>
								</tr>
							</thead>
							
							<tbody>
								<tr>
									<td>Links</td>
									<td>$ID</td>
								</tr>
								<tr>
									<td>Category</td>
									<td>$ID</td>
								</tr>
								<tr>
									<td>Reviews</td>
									<td>$ReviewID</td>
								</tr>
							</tbody>
						</table>
						
						So an example for the "Reviews" table would be:
						
						<pre><%Plugins::Thumb_Images::get_path(<span class="muted">$ReviewID</span>,"<span class="text-error">field_name</span>","<span class="text-warning">small</span>","<span class="text-info">Reviews</span>","<span class="text-success">url</span>")%></pre>
						
						...to get the original sized one you would use:
						
						<pre><%Plugins::Thumb_Images::get_path(<span class="muted">$ReviewID</span>,"<span class="text-error">field_name</span>","","<span class="text-info">Reviews</span>","<span class="text-success">url</span>")%></pre>
						
						(i.e just leave the 3rd option as blank)
						
						The options you need to pass are defined more below:
						
						<table class="table table-striped">
							<thead>
								<tr>
									<th>Option name</th>
									<th>Value to use</th>
								</tr>
							</thead>
							<tbody>
								<tr>
									<td><span class="muted">$ID</span></td>
									<td>Depending if on whiuch table you are using, this needs to be the ID of the record we are working with. So if you are working on detailed.html or link.html and want to show the image for a link, this would be $ID. Only reviews uses a different value!</td>
								</tr>
								<tr>
									<td><span class="text-error">field_name</span></td>
									<td>This is the name of the field you want to show</td>
								</tr>
								<tr>
									<td><span class="text-warning">size</span></td>
									<td>This is the size of the image you want to show. If you have for example a record for links:
										<b>Image1::300x300::medium</b>, and you wanted to show that image.. you would put "medium" in this setting
									</td>
								</tr>

								<tr>
									<td><span class="text-info">Table</span></td>
									<td>This is the table name you want to grab from. This is either: <b>Links</b>, <b>Category</b> or <b>Reviews</b>
									</td>
								</tr>

								<tr>
									<td><span class="text-success">what to get</span></td>
									<td>This is what you want the function to grab. Most people would just use "<b>url</b>" here, but if you want to get the FULL path (this is NOT a relative path!) for use in other code, you can use  "<b>path</b>" in this option.</td>
								</tr>								
								
							</tbody>
						</table>
						

						
						<h3>Support</h3>
						<a name="support"> </a>
						
						<p>If you need support, you can use the <a href="http://www.gossamer-threads.com/forum/" target="_blank">Gossamer Threads forum</a>, or email me on <a href="mailto:andy@ultranerds.co.uk">andy@ultranerds.co.uk</a></p>
						
					</div>
				</div>
				<br /><br />
			
			</div>
		
		
			<div class="navbar navbar-fixed-bottom" style="text-align: center; background: #eee; padding: 3px;"><b>Thumb_Images</b> v7.0.1. &copy; <b>Ultranerds</b>, 2000-2013. All Rights Reserved.</div>
		
		</body>
	
		
		
    </html>
	
	
	<script src="//netdna.bootstrapcdn.com/twitter-bootstrap/2.2.2/js/bootstrap.min.js"></script>
	
	|;
	
}


# Always end with a 1.
1;